@@ -1,6 +1,6 @@
 package OpenInteract2::Action::Box;
 
-# $Id: Box.pm,v 1.4 2003/06/25 14:10:48 lachoy Exp $
+# $Id: Box.pm,v 1.8 2004/05/22 01:51:18 lachoy Exp $
 
 use strict;
 use base qw( OpenInteract2::Action );
@@ -8,8 +8,11 @@ use Log::Log4perl            qw( get_logger );
 use Data::Dumper             qw( Dumper );
 use OpenInteract2::Constants qw( :log );
 use OpenInteract2::Context   qw( CTX );
+use OpenInteract2::Exception qw( oi_error );
 
-$OpenInteract2::Action::Box::VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::Action::Box::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 my @BOX_KEYS = qw( title weight base_template image image_alt );
 
@@ -22,7 +25,7 @@ my $MAX_WEIGHT     = 100;
 
 sub process_boxes {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     $BOX_CONFIG ||= CTX->server_config->{box};
 
@@ -52,14 +55,14 @@ sub process_boxes {
 
 sub _create_system_boxes {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     unless ( $SYSTEM_REQUIRED ) {
         $SYSTEM_BOX_CLASS = $BOX_CONFIG->{system_box_handler};
         eval "require $SYSTEM_BOX_CLASS";
         if ( $@ ) {
             $log->error( "FAILED: cannot include system box class ",
-                         "[$SYSTEM_BOX_CLASS]: $@" );
+                         "'$SYSTEM_BOX_CLASS': $@" );
             $SYSTEM_BOX_CLASS = undef;
             return ();
         }
@@ -86,7 +89,7 @@ sub _create_system_boxes {
 
 sub _create_custom_boxes {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     unless ( $CUSTOM_REQUIRED ) {
         $CUSTOM_BOX_CLASS = $BOX_CONFIG->{custom_box_handler};
@@ -94,7 +97,7 @@ sub _create_custom_boxes {
             eval "require $CUSTOM_BOX_CLASS";
             if ( $@ ) {
                 $log->error( "FAILED: cannot require custom box class ",
-                             "[$CUSTOM_BOX_CLASS]: $@" );
+                             "'$CUSTOM_BOX_CLASS': $@" );
                 return ();
             }
             else {
@@ -121,7 +124,7 @@ sub _create_custom_boxes {
 
 sub _create_box_actions {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my @actions = ();
 
@@ -138,9 +141,18 @@ BOX:
         # Now lookup box action
 
         my $box_action = eval { CTX->lookup_action( $use_info->{name} ) };
-        if ( $@ ) {
-            $log->warn( "Skipping box [$use_info->{name}] not found ",
-                        "in the action table" );
+        if ( $@ and $use_info->{is_template} eq 'yes' ) {
+            $box_action = eval { CTX->lookup_action( 'template_only' ) };
+            if ( $@ ) {
+                oi_error "Cannot create 'template_only' action -- this ",
+                         "should be in the 'base' package but I could not ",
+                         "find it."
+            }
+            $box_action->param( template => $use_info->{name} );
+        }
+        elsif ( $@ ) {
+            $log->warn( "Skipping box '$use_info->{name}', not found in ",
+                        "the action table and 'is_template' not set to 'yes'" );
             next BOX;
         }
 
@@ -150,6 +162,15 @@ BOX:
         foreach my $box_key ( @BOX_KEYS ) {
             next unless ( $use_info->{ $box_key } );
             $box_action->param( $box_key, $use_info->{ $box_key } );
+            $log->is_debug &&
+                $log->debug( "Adding box_key parameter to box: '$box_key' '$use_info->{ $box_key }'" );
+            delete $use_info->{ $box_key };
+        }
+
+        foreach my $param_name ( keys %{ $use_info->{params} } ) {
+            $log->is_debug &&
+                $log->debug( "Adding parameter to box: '$param_name' '$use_info->{params}{ $param_name }'" );
+            $box_action->param( $param_name, $use_info->{params}{ $param_name } );
         }
 
         # Assign default weight if not already there and if the weight
@@ -159,14 +180,14 @@ BOX:
             $box_action->param( 'weight', $DEFAULT_WEIGHT );
         }
         if ( $box_action->param( 'weight' ) > $MAX_WEIGHT ) {
-            $log->warn( "Skipping box [$use_info->{name}] since ",
-                          "its weight is more than ", $MAX_WEIGHT );
+            $log->warn( "Skipping box '$use_info->{name}' since ",
+                        "its weight is more than the max of ", $MAX_WEIGHT );
             next BOX;
         }
 
         $log->is_debug &&
-            $log->debug( "Putting box [$use_info->{name}] onto the",
-                         "stack with weight [$use_info->{weight}]" );
+            $log->debug( "Putting box '$use_info->{name}' onto the",
+                         "stack with weight '$use_info->{weight}'" );
         push @actions, $box_action;
     }
     return @actions;
@@ -176,18 +197,18 @@ BOX:
 
 sub _get_shell_template {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $box_template_name = CTX->request->theme_values->{box_template}
                             || $BOX_CONFIG->{default_template}
                             || $self->param( 'box_default_template' );
     unless ( $box_template_name =~ /::/ ) {
-        $log->warn( "Box shell [$box_template_name] is not a ",
+        $log->warn( "Box shell '$box_template_name' is not a ",
                     "valid package::name spec; using naked boxes" );
         $box_template_name = undef;
     }
     $log->is_debug &&
-        $log->debug( "Using box shell template [$box_template_name]" );
+        $log->debug( "Using box shell template '$box_template_name'" );
     return $box_template_name;
 }
 
@@ -196,7 +217,7 @@ sub _get_shell_template {
 
 sub _generate_box_content {
     my ( $self, $actions, $shell_template ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my @content = ();
     $log->is_debug &&
@@ -556,7 +577,7 @@ L<OpenInteract::Theme|OpenInteract::Theme>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Chris Winters. All rights reserved.
+Copyright (c) 2001-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
