@@ -1,6 +1,6 @@
 package OpenInteract2::Action::Group;
 
-# $Id: Group.pm,v 1.7 2003/06/25 14:10:49 lachoy Exp $
+# $Id: Group.pm,v 1.8 2004/02/18 05:25:23 lachoy Exp $
 
 use strict;
 use base qw( OpenInteract2::Action::CommonAdd
@@ -13,13 +13,15 @@ use OpenInteract2::Context   qw( CTX );
 use SPOPS::Secure            qw( :level );
 use SPOPS::Utility;
 
-$OpenInteract2::Action::Group::VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::Action::Group::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 use constant MEMBER_FIELD => 'group_members';
 
 sub list {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
     $log->is_info &&
         $log->info( "Getting list of groups" );
     my $groups = eval {
@@ -69,7 +71,7 @@ sub _display_form_customize {
 
 sub _get_all_users {
     my ( $self, $members ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my @sorted_nonmember_info = ();
     my $all_users = eval {
@@ -95,7 +97,7 @@ sub _get_all_users {
 
 sub _get_group_users {
     my ( $self, $group ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $members = eval { $group->user };
     if ( $@ ) {
@@ -127,7 +129,7 @@ sub _update_post_action {
 
 sub _post_save {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $group = $self->param( 'c_object' );
     my $request = CTX->request;
@@ -171,4 +173,4 @@ sub _post_save {
     return undef;
 }
 
-1;
+1;
\ No newline at end of file
