@@ -1,6 +1,6 @@
 package OpenInteract2::Action::LookupEdit;
 
-# $Id: LookupEdit.pm,v 1.8 2003/06/25 14:10:50 lachoy Exp $
+# $Id: LookupEdit.pm,v 1.10 2004/02/18 05:23:09 lachoy Exp $
 
 # See 'doc/lookup.pod' for description of the fields in the action
 # table we use.
@@ -12,7 +12,9 @@ use OpenInteract2::Constants qw( :log );
 use OpenInteract2::Context   qw( CTX );
 use SPOPS::Secure            qw( :level );
 
-$OpenInteract2::Action::LookupEdit::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::Action::LookupEdit::VERSION = sprintf("%d.%02d", q$Revision: 1.10 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 my $NEW_KEY     = '_new_';
 my $REMOVE_KEY  = '_remove_';
@@ -87,7 +89,7 @@ sub partition_listing {
 
 sub listing {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
     my $lookup_type = $request->param( 'lookup_type' );
@@ -166,7 +168,7 @@ sub listing {
 
 sub edit {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
     my $lookup_type  = $request->param( 'lookup_type' );
@@ -223,7 +225,7 @@ sub _retrieve_id_list {
 
 sub _find_all_lookups {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $action_table = CTX->action_table;
     my ( @lookup_list );
@@ -254,7 +256,7 @@ sub _find_distinct_values {
 
 sub _find_lookup_info {
     my ( $self, $lookup_type ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     unless ( $lookup_type ) {
         return ( undef,  'Cannot list lookup entries without a lookup type.' );
@@ -275,7 +277,7 @@ sub _find_lookup_info {
 
 sub _lookup_entries {
     my ( $self, $lookup_info, $partition_value ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $lookup_object_key = $lookup_info->{object_key};
     my $lookup_class = CTX->lookup_object( $lookup_object_key );
@@ -303,7 +305,7 @@ sub _lookup_related_objects {
 
 sub _persist {
     my ( $self, $lookup_class, $field_list, $id ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
     my $is_new = ( $id =~ /^$NEW_KEY/ );
@@ -455,11 +457,11 @@ Nothing known.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Chris Winters. All rights reserved.
+Copyright (c) 2001-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =head1 AUTHORS
 
-Chris Winters E<lt>chris@cwinters.comE<gt>
+Chris Winters E<lt>chris@cwinters.comE<gt>
\ No newline at end of file
