@@ -1,6 +1,6 @@
 package OpenInteract2::Action::NewUser;
 
-# $Id: NewUser.pm,v 1.9 2003/06/25 14:10:50 lachoy Exp $
+# $Id: NewUser.pm,v 1.19 2004/05/22 15:43:23 lachoy Exp $
 
 use strict;
 use base qw( OpenInteract2::Action );
@@ -10,7 +10,9 @@ use OpenInteract2::Context   qw( CTX );
 use SPOPS::Secure qw( :level :scope );
 use SPOPS::Utility;
 
-$OpenInteract2::Action::NewUser::VERSION = sprintf("%d.%02d", q$Revision: 1.9 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::Action::NewUser::VERSION = sprintf("%d.%02d", q$Revision: 1.19 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 use constant REMOVAL_TIME => 60 * 60 * 24; # 1 day
 
@@ -30,12 +32,12 @@ sub add {
     my ( $plain_pass, $crypted_pass ) = $self->_create_password;
     my $new_user = $self->_create_new_user( $login, $email, $crypted_pass );
     $self->_add_new_user_metadata( $new_user );
-    $self->_send_user_email( $new_user, $plain_pass );
+    $self->_send_new_user_email( $new_user, $plain_pass );
 
-    # TODO: Create method in OI2::Session set_user_id() so we don't
+    # TODO: Create method in OI2::Session get/set_user_id() so we don't
     # have to know the 'user_id' key (??)
 
-    $self->response->return_url( $self->create_url({ TASK => '' }) );
+    CTX->response->return_url( $self->create_url({ TASK => '' }) );
     if ( $self->param( 'autologin' ) eq 'yes' ) {
         $request->auth_user( $new_user );
         $request->auth_is_logged_in(1);
@@ -48,18 +50,18 @@ sub add {
 
 sub _validate_login_and_email {
     my ( $self, $login, $email ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     unless ( $self->_validate_email( $email ) ) {
-        my $msg = 'The email address you entered is invalid.';
-        $self->param_add( error_msg => $msg );
+        $self->param_add(
+            error_msg => $self->_msg( 'base_user.new.invalid_email' ) );
         $self->param( requested_login => $login );
         die $self->execute({ task => 'display' });
     }
 
     unless ( $login ) {
-        my $msg = 'You cannot create an account without entering a login!';
-        $self->param_add( error_msg => $msg );
+        $self->param_add(
+            error_msg => $self->_msg( 'base_user.new.no_login' ) );
         $self->param( working_email => $email );
         die $self->execute({ task => 'display' });
     }
@@ -73,18 +75,33 @@ sub _validate_login_and_email {
         $log->error( "Error fetching dupecheck user: $@" );
     }
     if ( $user ) {
-        my $msg = 'The username you requested is already in use.';
-        $self->param_add( error_msg => $msg );
+        $self->param_add(
+            error_msg => $self->_msg( 'base_user.new.name_in_use' ) );
         $self->param( working_email => $email );
-        die $self->execute({ task => 'display' });
+        die $self->execute({ task => 'display' }), "\n";
     }
+
+    $user = eval {
+        $user_class->fetch_by_email( $email, { skip_security => 1,
+                                               return_single => 1 } )
+    };
+    if ( $@ ) {
+        $log->error( "Error fetching dupecheck user: $@" );
+    }
+    if ( $user ) {
+        $self->param_add(
+            error_msg => $self->_msg( 'base_user.new.duplicate_email' ) );
+        $self->param( working_email => $email );
+        die $self->execute({ task => 'display' }), "\n";
+    }
+
     return undef;
 }
 
 sub _validate_email {
     my ( $self, $email ) = @_;
     return undef unless ( $email );
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     eval "require Email::Valid";
     unless ( $@ ) {
@@ -95,14 +112,14 @@ sub _validate_email {
 
     $log->is_info &&
         $log->info( "Email::Valid NOT loaded, trying ",
-                         "Mail::RFC822::Address" );
+                    "Mail::RFC822::Address" );
     eval "require Mail::RFC822::Address";
     if ( $@ ) {
         $log->error( "Email::Valid NOT loaded and Mail::RFC822::Address ",
                      "NOT loaded , cannot validate email. (You should ",
                      "not have been able to install this package...)" );
-        my $msg = 'Insufficient modules to check email validity.';
-        $self->param_add( error_msg => $msg );
+        $self->param_add(
+            error_msg => $self->_msg( 'base_user.new.no_mailcheck_module' ) );
         die $self->execute({ task => 'display' });
     }
     return Mail::RFC822::Address::valid( $email );
@@ -111,14 +128,14 @@ sub _validate_email {
 sub _create_password {
     my ( $self ) = @_;
     my $plain = SPOPS::Utility->generate_random_code( 12, 'mixed' );
-    my $crypted = ( CTX->server_config->{login}{crypt_password} )
+    my $crypted = ( CTX->lookup_login_config->{crypt_password} )
                     ? SPOPS::Utility->crypt_it( $plain ) : $plain;
     return ( $plain, $crypted );
 }
 
 sub _create_new_user {
     my ( $self, $login, $email, $password ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     # Now, create an entry in the user table; note that we set the removal
     # date to now plus whatever REMOVAL_TIME is set to
@@ -127,13 +144,14 @@ sub _create_new_user {
     $new_user->{login_name} = $login;
     $new_user->{email}      = $email;
     $new_user->{password}   = $password;
-    $new_user->{theme_id}   = CTX->server_config->{default_objects}{theme};
+    $new_user->{theme_id}   = CTX->lookup_default_object_id( 'theme' );
     $new_user->{removal_date} = OpenInteract2::Util->now(
                                         { time => time + REMOVAL_TIME });
     eval { $new_user->save };
     if ( $@ ) {
         $log->error( "Failure to create new user: $@" );
-        $self->param_add( error_msg => "Failed to create your record: $@" );
+        $self->param_add(
+            error_msg => $self->_msg( 'base_user.new.create_failure', $@ ) );
         die $self->execute({ task => 'display' });
     }
     return $new_user;
@@ -141,25 +159,27 @@ sub _create_new_user {
 
 sub _add_new_user_metadata {
     my ( $self, $new_user ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     # Ensure that the user can read/write his/her own record!
 
-    eval { $new_user->set_item_security(
-                              { class     => ref $new_user,
-                                object_id => $new_user->id,
-                                scope     => SEC_SCOPE_USER,
-                                scope_id  => $new_user->id,
-                                level     => SEC_LEVEL_WRITE }) };
+    eval {
+        $new_user->set_item_security(
+            { class     => ref( $new_user ),
+              object_id => $new_user->id,
+              scope     => SEC_SCOPE_USER,
+              scope_id  => $new_user->id,
+              level     => SEC_LEVEL_WRITE })
+       };
 
     # Log the failed security set, if it happens...
 
     if ( $@ ) {
         $log->error( "Failed to set security so that new user ",
-                     "[$new_user->{login_name}] can see her record: $@" );
-        my $msg = "Error setting security information on your record: $@";
-        $self->param_add( error_msg => $msg );
-        die $self->execute({ task => 'display' });
+                     "'$new_user->{login_name}' can see her record: $@" );
+        $self->param_add(
+            error_msg => $self->_msg( 'base_user.new.security_failure', $@ ) );
+        die $self->execute({ task => 'display' }), "\n";
     }
 
     # ...otherwise, mark the user as the creator of his/her own record
@@ -169,31 +189,35 @@ sub _add_new_user_metadata {
                                  scalar( $new_user->id ) );
 }
 
+# If that worked ok, send the user an email with the password created
+
 sub _send_new_user_email {
     my ( $self, $new_user, $plain_password ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
     my $server_name = $request->server_name;
 
-    # If that worked ok, send the user an email with the password created
-
     my %email_params = ( login       => $new_user->{login_name},
                          password    => $plain_password,
                          server_name => $server_name );
+    $log->info( "Sending email to '$email_params{login}' for server ",
+                "'$email_params{server_name}'" );
     my $message = $self->generate_content(
                               \%email_params,
                               { name => 'base_user::new_user_email' } );
-    eval { OpenInteract2::Util->send_email({
-                message => $message,
-                to      => $new_user->{email},
-                subject => "Account information for $server_name" }) };
+    my $subject = $self->_msg( 'base_user.new_mail.subject', $server_name );
+    eval {
+        OpenInteract2::Util->send_email(
+            { message => $message,
+              to      => $new_user->{email},
+              subject => $subject })
+    };
     if ( $@ ) {
         $log->error( "Cannot send email! $@" );
-        $self->param_msg(
-            error_msg => "Could not send email to you with generated " .
-                         "password: $@" );
-        die $self->execute({ task => 'display' });
+        $self->param_add(
+            error_msg => $self->_msg( 'base_user.new.mail_failure', $@ ) );
+        die $self->execute({ task => 'display' }), "\n";
     }
 }
 
@@ -242,7 +266,7 @@ None known
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Chris Winters. All rights reserved.
+Copyright (c) 2001-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
