@@ -1,6 +1,6 @@
 package OpenInteract2::Action::Package;
 
-# $Id: Package.pm,v 1.7 2003/06/25 14:10:48 lachoy Exp $
+# $Id: Package.pm,v 1.10 2004/02/18 05:25:22 lachoy Exp $
 
 use strict;
 use base qw( OpenInteract2::Action );
@@ -8,7 +8,9 @@ use Log::Log4perl            qw( get_logger );
 use OpenInteract2::Constants qw( :log );
 use OpenInteract2::Context   qw( CTX );
 
-$OpenInteract2::Action::Package::VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::Action::Package::VERSION = sprintf("%d.%02d", q$Revision: 1.10 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 sub listing {
     my ( $self ) = @_;
@@ -20,16 +22,16 @@ sub listing {
 
 sub show {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
     my $repository = CTX->repository;
     my $pkg = $self->param( 'package' );
-    $pkg = undef unless ( ref $pkg eq 'OpenInteract2::Package' );
+    $pkg = undef unless ( ref $pkg && $pkg->isa( 'OpenInteract2::Package' ) );
     unless ( $pkg ) {
         my $name = CTX->request->param( 'name' );
-        $pkg = $repository->fetch_package_by_name({ name => $name });
+        $pkg = $repository->fetch_package( $name );
         unless ( $pkg ) {
-            $log->warn( "Failed to find package [$name]" );
-            my $error_msg = "Could not find package with name [$name]";
+            $log->warn( "Failed to find package '$name'" );
+            my $error_msg = "Could not find package with name '$name'";
             $self->param_add( error_msg => $error_msg );
             return $self->execute({ task => 'listing' });
         }
@@ -92,7 +94,7 @@ Name of the package you want to display.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Chris Winters. All rights reserved.
+Copyright (c) 2001-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
@@ -100,4 +102,3 @@ it under the same terms as Perl itself.
 =head1 AUTHORS
 
 Chris Winters E<lt>chris@cwinters.comE<gt>
-
