@@ -1,6 +1,6 @@
 package OpenInteract2::Action::Page;
 
-# $Id: Page.pm,v 1.12 2003/09/04 03:37:22 lachoy Exp $
+# $Id: Page.pm,v 1.18 2004/05/22 15:02:58 lachoy Exp $
 
 use strict;
 use base qw( OpenInteract2::Action::CommonSearch
@@ -17,7 +17,9 @@ use OpenInteract2::Context   qw( CTX );
 use OpenInteract2::Exception qw( oi_error );
 use SPOPS::Secure            qw( SEC_LEVEL_WRITE );
 
-$OpenInteract2::Action::Page::VERSION = sprintf("%d.%02d", q$Revision: 1.12 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::Action::Page::VERSION = sprintf("%d.%02d", q$Revision: 1.18 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 # 52 weeks -- default expiration for page
 
@@ -34,9 +36,8 @@ my %DISPLAY_TYPES = map { $_ => 1 } ( 'text/html' );
 my $DEFAULT_PAGE_SEPARATOR = '<!--PAGE-->';
 
 sub admin {
-    my ( $self, @params ) = @_;
-    $self->task( 'actions' );
-    return $self->execute( @params );
+    my ( $self ) = @_;
+    return $self->({ task => 'actions' });
 }
 
 sub actions {
@@ -62,11 +63,11 @@ sub help {
 
 sub _get_task {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
     my ( $task );
-    if ( $request->url_relative !~ /^page$/i ) {
+    if ( $request->url_relative !~ m|^page/|i ) {
         $task = 'display';
     }
     else {
@@ -74,7 +75,7 @@ sub _get_task {
                 || $self->task_default;
     }
     $log->is_debug &&
-        $log->debug( "Task found from _get_task: [$task]" );
+        $log->debug( "Task found from _get_task: '$task'" );
     return $task;
 }
 
@@ -86,7 +87,7 @@ sub _search_criteria_customize {
     # Administrators can pick whether to find active pages or not,
     # everyone else has to only search active pages. (Note: don't use
     # '_read_field_toggled' here because we want to ignore the
-    # parameter if it wasn't chosen.
+    # parameter if it wasn't chosen rather than use a 'no' value.
 
     my $request = CTX->request;
     if ( $request->auth_is_admin ) {
@@ -102,11 +103,11 @@ sub _search_criteria_customize {
 # Retrieve all directories, expanding the one we were asked to (if at
 # all). Note that these are just the objects in the database, although
 # there should be a corresponding content entry for every one of these
-# in the filesystem or database.
+# in the filesystem (or database if you're a crazy nut).
 
 sub directory_list {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $page_class = CTX->lookup_object( 'page' );
     my $selected_dir = CTX->request->param( 'selected_dir' );
@@ -138,7 +139,7 @@ sub directory_list {
 
 sub specify_subdirectory {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $parent_dir = CTX->request->param( 'parent_directory' );
     unless ( $self->_check_location_writable( $parent_dir ) ) {
@@ -159,7 +160,7 @@ sub specify_subdirectory {
 
 sub add_subdirectory {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
     my $parent_dir = $request->param( 'parent_directory' );
@@ -183,7 +184,7 @@ sub add_subdirectory {
                    created_directory => $display_path,
                    action            => 'add_subdirectory' );
 
-    my $full_dir = File::Spec->catdir( CTX->server_config->{dir}{html},
+    my $full_dir = File::Spec->catdir( CTX->lookup_directory( 'html' ),
                                        $parent_dir, $cleaned_dir );
     eval { File::Path::mkpath( $full_dir, undef, 0775 ) };
     if ( $@ ) {
@@ -206,7 +207,7 @@ sub add_subdirectory {
 
 sub remove_directory {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
     my $page_class = CTX->lookup_object( 'page' );
@@ -268,7 +269,7 @@ sub remove_directory {
             $log->is_debug &&
                 $log->debug( "Removed page [$page->{location}] ok" );
         }
-        my $full_dir = File::Spec->catdir( CTX->server_config->{dir}{html},
+        my $full_dir = File::Spec->catdir( CTX->lookup_directory( 'html' ),
                                            $directory );
         $log->is_debug &&
             $log->debug( "Removing directory [$full_dir]" );
@@ -307,7 +308,7 @@ sub remove_directory {
 
 sub specify_rename {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $location = CTX->request->param( 'location' );
     unless ( $self->_check_location_writable( $location ) ) {
@@ -334,7 +335,7 @@ sub specify_rename {
 
 sub rename_file {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
     my $page_class = CTX->lookup_object( 'page' );
@@ -420,7 +421,7 @@ sub rename_file {
 
 sub _check_location_writable {
     my ( $self, $location ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
     my $level = eval {
@@ -441,7 +442,7 @@ sub _check_location_writable {
 
 sub fetch_object {
     my ( $self, $id, @id_field ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $existing_location = CTX->request->param( 'old_location' );
     my $fetch_id = ( $id =~ m|^/| ) ? $id : "/$id";
@@ -456,7 +457,7 @@ sub fetch_object {
 
 sub _modify_check_upload {
     my ( $self, $page ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
 
@@ -468,43 +469,52 @@ sub _modify_check_upload {
     if ( $has_upload eq 'yes' ) {
         $log->is_debug &&
             $log->debug( "Handling file upload" );
-        my ( $type, $msg ) = $self->_handle_uploaded_file( $page );
-        return ( $type, $msg ) if ( $type );
+        $self->_handle_uploaded_file( $page );
+        return 1;
     }
+    return 0;
 }
 
 # Yes, this is out of the normal order. It's just that show() is so
 # big and includes so much stuff...
 
+sub _add_customize {
+    my ( $self, $page, $add_params ) = @_;
+    $self->_save_customize( $page, $add_params );
+}
+
 sub _update_customize {
     my ( $self, $page, $old_data, $update_params ) = @_;
-    my $log = get_logger( LOG_APP );
+    $self->_save_customize( $page, $update_params );
+}
+
+sub _save_customize {
+    my ( $self, $page, $save_params ) = @_;
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
 
-    # TODO: this isn't correct return value
-    my @info = $self->_modify_check_upload( $page );
-    return @info if ( scalar @info );
+    my $handled = $self->_modify_check_upload( $page );
+    return if ( $handled );
 
     # See if we're using the 'simple' form, and if so set the location
     # of the file from the filename uploaded (after cleaning)
 
     my $page_class = CTX->lookup_object( 'page' );
-    my %opts = ();
+
     my $form_type = $request->param( 'form_type' ) || 'normal';
     if ( $form_type eq 'simple' ) {
         my $dir = $request->param( 'directory' );
         unless ( $dir ) {
-            my $error = "Failed to upload file. No directory specified.";
-            return ( OI_ERROR, $error );
+            oi_error "Failed to upload file. No directory specified.";
         }
         $dir =~ s|/$||;
         my $filename = $page_class->clean_filename(
                                    $page->{tmp_upload}{filename} );
         $page->{location} = join( '/', $dir, $filename );
         $log->is_debug &&
-            $log->debug( "Set location to [$page->{location}] from",
-                         "filename reported by client [$filename]" );
+            $log->debug( "Set location to '$page->{location}' from",
+                         "filename reported by client '$filename'" );
 
         # Check to see if this location is already in the
         # database. (For non-simple pages this is done in the normal
@@ -519,31 +529,13 @@ sub _update_customize {
         }
     }
 
-    # If we're not using the simple form, check to see if the user
-    # changed the location, then we need to set the ID so the UPDATE
-    # works properly.
-
-    else {
-        unless ( $page->{location} =~ m|^/| ) {
-            $page->{location} = "/$page->{location}";
-        }
-        if ( $old_data->{location} and
-            $page->{location} ne $old_data->{location} ) {
-            $opts{use_id} = $old_data->{location};
-            $log->is_debug &&
-                $log->debug( "This should be an UPDATE where ",
-                             "we change the primary key from  ",
-                             "[$old_data->{location}] to [$page->{location}]" );
-        }
-    }
-
     # Ensure that the location is clean
 
     $log->is_debug &&
-        $log->debug( "Location before clean [$page->{location}]" );
+        $log->debug( "Location before clean '$page->{location}'" );
     $page->{location} = $page_class->clean_location( $page->{location} );
     $log->is_debug &&
-        $log->debug( "Location after clean [$page->{location}]" );
+        $log->debug( "Location after clean '$page->{location}'" );
 
     $page->{expires_on} ||= DateTime->now + DEFAULT_EXPIRE;
     $page->{mime_type} ||= 'text/html';
@@ -554,28 +546,28 @@ sub _update_customize {
 
     unless ( $self->_is_displayable( $page->{mime_type} ) ) {
         $log->is_debug &&
-            $log->debug( "Not displayable [$page->{mime_type}];",
+            $log->debug( "Not displayable '$page->{mime_type}';",
                          "don't scan content for index" );
         $page->{is_file} = 'yes';
         if ( $page->CONFIG->{fulltext_field} ) {
-            $opts{fulltext_field} = [ grep ! /^content$/,
-                                             @{ $page->{fulltext_field} } ];
+            $save_params->{fulltext_field} =
+                [ grep ! /^content$/, @{ $page->{fulltext_field} } ];
         }
     }
-
-    return ( OI_OK, \%opts );
 }
 
 # If this is a successful update and the item's location has been
 # changed, we need to tell the content implementation class to rename
 # the content and do whatever other actions it requires.
 
-sub _edit_post_action_customize {
-    my ( $self, $page, $old_data ) = @_;
+sub _update_post_action {
+    my ( $self ) = @_;
+    my $page = $self->param( 'c_object' );
+    my $old_data = $self->param( 'c_object_old_data' );
     if ( $page->{location} ne $old_data->{location} ) {
         $page->rename_content( $old_data->{location} );
     }
-    return 1;
+    return;
 }
 
 
@@ -595,7 +587,7 @@ sub display_remove_status {
 
 sub display_modify_status {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my %params = ();
     my $page = $self->param( 'c_object' );
@@ -636,7 +628,7 @@ sub display_form {
 
 sub _show {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     $log->is_debug &&
         $log->debug( "Edit status of page: ",
@@ -682,9 +674,9 @@ sub _show {
     my $do_edit = $self->param( 'edit' )
                   || CTX->request->param( 'edit' );
     unless ( $page or $do_edit ) {
-        $self->response->content_type( 'text/html' );
+        CTX->response->content_type( 'text/html' );
         $location ||= $page->{location};
-        $log->warn( "Page for [$location] doesn't exist; did not specify ",
+        $log->warn( "Page for '$location' doesn't exist; did not specify ",
                     "edit mode. Bail." );
         return $self->generate_content({ url => $location },
                                        { name => 'error_not_found' } );
@@ -711,7 +703,7 @@ sub _show {
 
 sub _show_displayable_page {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $page = $self->param( 'page' );
     unless ( ref $page ) {
@@ -732,11 +724,12 @@ sub _show_displayable_page {
     while ( $page->{storage} eq 'alias' ) {
         $log->is_debug &&
             $log->debug( "Trying to find aliased content from ",
-                         "location [$page->{content_location}]" );
-        $page = eval { $page->page_by_location(
-                                   $page->{content_location} ) };
+                         "location '$page->{content_location}'" );
+        $page = eval {
+            $page->page_by_location( $page->{content_location} )
+        };
         if ( $@ or ! $page ) {
-            $log->error( "Location for alias [$page->{content_location}]",
+            $log->error( "Location for alias '$page->{content_location}'",
                          "wasn't found: $@" );
             my %params = ( location       => $page->{location},
                            alias_location => $page->{content_location} );
@@ -747,8 +740,11 @@ sub _show_displayable_page {
     }
 
     $log->is_debug &&
-        $log->debug( "Display [$page->{location}] as normal HTML" );
-    $page->fetch_content;
+        $log->debug( "Display '$page->{location}' as normal HTML" );
+    eval { $page->fetch_content };
+    if ( $@ ) {
+        return "Failed to load content: $@";
+    }
 
     # Use page metadata to modify display
 
@@ -773,7 +769,7 @@ sub _show_displayable_page {
 
 sub _show_editable_page {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $page = $self->param( 'page' )
                || CTX->lookup_object( 'page' )->new;
@@ -813,13 +809,12 @@ sub _show_editable_page {
 
 sub _show_nondisplayable_page {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
-    my $response = $self->response;
     my $page     = $self->param( 'page' );
-    my $full_filename = join( '', CTX->server_config->{dir}{html},
+    my $full_filename = join( '', CTX->lookup_directory( 'html' ),
                                   $page->{location} );
-    $response->send_file( $full_filename );
+    CTX->response->send_file( $full_filename );
     $log->is_debug &&
         $log->debug( "File being retrieved is not directly",
                      "displayable. Set 'send_file' to [$page->{location}]" );
@@ -829,9 +824,9 @@ sub _show_nondisplayable_page {
 
 sub _fail_page_fetch {
     my ( $self, $location, $error ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
-    $self->response->content_type( 'text/html' );
+    CTX->response->content_type( 'text/html' );
     $log->is_debug &&
         $log->debug( "Could not retrieve page. Error [$error]" );
     if ( $error->isa( 'OpenInteract2::Exception::Security' ) ) {
@@ -940,7 +935,7 @@ PCOUNT
 
 sub _add_object_boxes {
     my ( $self, $page, $p ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $box_string = $page->{boxes};
 
@@ -978,7 +973,7 @@ sub _add_object_boxes {
 
 sub _is_active {
     my ( $self, $page ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     if ( $page->{is_active} eq 'no' ) {
         $log->is_debug &&
@@ -1024,16 +1019,16 @@ sub _is_active {
 
 sub _handle_uploaded_file {
     my ( $self, $page ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     $log->is_debug &&
         $log->debug( "User is requesting content from uploaded file" );
     my $upload  = CTX->request->upload( 'content_upload' );
     unless ( $upload ) {
         my $error_msg = 'You checked off that you wanted to upload a ' .
-                        'file but did not upload one. Why do you tease so?';
-        my %error_opts = ( method => 'show', error_msg => $error_msg, edit => 1 );
-        return ( OI_ERROR, \%error_opts );
+                        'file but did not upload one. Why do you tease ' .
+                        'me like that?';
+        die $error_msg;
     }
     $page->{tmp_upload} = { filename => $upload->filename,
                             size     => $upload->size,
@@ -1108,17 +1103,13 @@ L<OpenInteract2::Action::Common|OpenInteract2::Action::Common>
 B<notify>: implemented in
 L<OpenInteract2::Action::Common|OpenInteract2::Action::Common>
 
-=head1 TO DO
-
-Nothing known.
-
 =head1 SEE ALSO
 
 L<OpenInteract2::Action::Common|OpenInteract2::Action::Common>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Chris Winters. All rights reserved.
+Copyright (c) 2001-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
