@@ -1,18 +1,20 @@
 # TODO: This is not an Action; should it be? Should it be here?
 package OpenInteract2::Action::SystemBoxes;
 
-# $Id: SystemBoxes.pm,v 1.5 2003/06/25 14:10:48 lachoy Exp $
+# $Id: SystemBoxes.pm,v 1.7 2004/02/18 05:25:22 lachoy Exp $
 
 use strict;
 use Log::Log4perl            qw( get_logger );
 use OpenInteract2::Constants qw( :log );
 use OpenInteract2::Context   qw( CTX );
 
-$OpenInteract2::Action::SystemBoxes::VERSION = sprintf("%d.%02d", q$Revision: 1.5 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::Action::SystemBoxes::VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 sub handler {
     my ( $class ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my @boxes = ();
     my $request = CTX->request;
@@ -106,11 +108,11 @@ L<OpenInteract2::Action::Box>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Chris Winters. All rights reserved.
+Copyright (c) 2001-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =head1 AUTHORS
 
-Chris Winters E<lt>chris@cwinters.comE<gt>
+Chris Winters E<lt>chris@cwinters.comE<gt>
\ No newline at end of file
