@@ -1,6 +1,6 @@
 package OpenInteract2::Action::Theme;
 
-# $Id: Theme.pm,v 1.7 2003/06/25 14:10:50 lachoy Exp $
+# $Id: Theme.pm,v 1.10 2004/02/18 05:23:09 lachoy Exp $
 
 use strict;
 use base qw( OpenInteract2::Action::CommonUpdate
@@ -11,13 +11,15 @@ use OpenInteract2::Constants qw( :log );
 use OpenInteract2::Context   qw( CTX );
 use SPOPS::Secure qw( :all );
 
-$OpenInteract2::Action::Theme::VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::Action::Theme::VERSION = sprintf("%d.%02d", q$Revision: 1.10 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 # List the available themes
 
 sub list {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     $log->is_info &&
         $log->info( "Listing themes in system" );
@@ -103,7 +105,7 @@ sub _update_post_action {
 
 sub _post_save {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     # Now work with the individual properties. First, set the theme_id
     # just in case we created a new one
@@ -273,17 +275,13 @@ B<remove>
 
 Remove a theme from the system.
 
-=head1 TO DO
-
-=head1 BUGS
-
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Chris Winters. All rights reserved.
+Copyright (c) 2001-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =head1 AUTHORS
 
-Chris Winters E<lt>chris@cwinters.comE<gt>
+Chris Winters E<lt>chris@cwinters.comE<gt>
\ No newline at end of file
