@@ -1,6 +1,6 @@
 package OpenInteract2::Action::User;
 
-# $Id: User.pm,v 1.11 2003/06/25 14:10:50 lachoy Exp $
+# $Id: User.pm,v 1.17 2004/02/22 04:45:52 lachoy Exp $
 
 use strict;
 use base qw( OpenInteract2::Action::CommonDisplay
@@ -13,7 +13,9 @@ use OpenInteract2::Context   qw( CTX );
 use SPOPS::Secure qw( :level :scope );
 use SPOPS::Utility;
 
-$OpenInteract2::Action::User::VERSION = sprintf("%d.%02d", q$Revision: 1.11 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::Action::User::VERSION = sprintf("%d.%02d", q$Revision: 1.17 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 sub search_form {
     my ( $self ) = @_;
@@ -28,7 +30,8 @@ sub search {
         CTX->lookup_object( 'user' )->fetch_iterator( $fetch_params )
     };
     if ( $@ ) {
-        $self->param_add( error_msg => "Error running search: $@" );
+        $self->param_add(
+            error_msg => $self->_msg( 'base_user.user.search_error', $@ ) );
     }
     return $self->generate_content(
                     { user_iterator => $iter },
@@ -87,24 +90,45 @@ sub _search_params_ldap {
 
 sub _display_add_customize {
     my ( $self, $template_params ) = @_;
-    $template_params->{user}->{theme_id} = CTX->request->theme->id;
+    $template_params->{user}{theme_id} = CTX->request->theme->id;
+    $self->_display_add_available_languages( $template_params );
     return undef;
 }
 
 sub _display_form_customize {
-    goto &_display_customize;
+    my ( $self, $template_params ) = @_;
+    $self->_display_add_available_languages( $template_params );
+    $self->_display_add_user_groups( $template_params );
+    return undef;
 }
 
 sub _display_customize {
+    my ( $self, $template_params ) = @_;
+    $self->_display_add_user_groups( $template_params );
+    return undef;
+}
+
+sub _display_add_user_groups {
     my ( $self, $template_params ) = @_;
     $template_params->{group_list} = eval {
         $template_params->{user}->group
     };
     if ( $@ ) {
         $self->param_add(
-            error_msg => "Failed to fetch user's groups: $@" );
+            error_msg => $self->_msg( 'base_user.user.group_fetch_fail', $@ ) );
+    }
+}
+
+sub _display_add_available_languages {
+    my ( $self, $template_params ) = @_;
+    $template_params->{language_list} = eval {
+        CTX->lookup_object( 'user_language' )
+            ->fetch_group({ order => 'language' });
+    };
+    if ( $@ ) {
+        $self->param_add(
+            error_msg => $self->_msg( 'base_user.user.lang_fetch_fail', $@ ) );
     }
-    return undef;
 }
 
 sub _add_customize {
@@ -120,7 +144,7 @@ sub _update_customize {
 
 sub _check_password_change {
     my ( $self, $user, $fail_task ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $request = CTX->request;
     my $password = $request->param( 'password' );
@@ -132,7 +156,7 @@ sub _check_password_change {
     my $password_confirm = $request->param( 'password_conf' );
     unless ( $password eq $password_confirm ) {
         $self->param_add(
-               error_msg => 'Passwords do not match. Please re-enter.' );
+               error_msg => $self->_msg( 'base_user.user.password_mismatch' ) );
         die $self->execute({ task => $fail_task });
     }
     $log->is_debug &&
@@ -141,7 +165,8 @@ sub _check_password_change {
         $password = SPOPS::Utility->crypt_it( $password );
     }
     $user->{password} = $password;
-    $self->param_add( status_msg => 'Password changed' );
+    $self->param_add(
+        status_msg => $self->_msg( 'base_user.user.password_changed' ) );
     return undef;
 }
 
@@ -152,7 +177,7 @@ sub _check_password_change {
 
 sub _add_post_action {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $user = $self->param( 'c_object' );
     eval {
@@ -164,9 +189,10 @@ sub _add_post_action {
     if ( $@ ) {
         $log->error( "Error modifying group membership: $@" );
         $self->param_add(
-                error_msg => 'Record created but group membership wrong.' );
+                error_msg => $self->_msg( 'base_user.user.group_add_fail' ) );
     }
-    $self->param_add( status_msg => "User added to 'public' group" );
+    $self->param_add(
+        status_msg => $self->_msg( 'base_user.user.group_add_ok' ) );
     return undef;
 }
 
