@@ -1,6 +1,6 @@
 package OpenInteract2::FullText;
 
-# $Id: FullText.pm,v 1.5 2003/09/03 13:40:10 lachoy Exp $
+# $Id: FullText.pm,v 1.8 2004/03/19 05:56:06 lachoy Exp $
 
 use strict;
 use DBI                      qw( SQL_VARCHAR SQL_INTEGER );
@@ -14,7 +14,9 @@ BEGIN {
     eval { require OpenInteract2::FullTextIterator };
 }
 
-$OpenInteract2::FullText::VERSION = sprintf("%d.%02d", q$Revision: 1.5 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::FullText::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 # TODO: Set this in the server config
 use constant DS_NAME => 'main';
@@ -46,7 +48,7 @@ my @SQL_FIELD_NAMES = qw/ object_key term occur /;
 
 sub create_class_index {
     my ( $class ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $iter = eval { $class->fetch_iterator };
     if ( $@ ) {
@@ -75,7 +77,7 @@ sub create_class_index {
 
 sub search_fulltext_index {
     my ( $class, $p ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     return [] unless ( scalar @{ $p->{search_terms} } );
 
@@ -161,7 +163,7 @@ sub _fulltext_object_listing {
 
 sub ruleset_factory {
     my ( $class, $rs_table ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $obj_class = ref $class || $class;
     push @{ $rs_table->{post_save_action} }, \&reindex_object;
@@ -177,14 +179,14 @@ sub ruleset_factory {
 
 sub reindex_object {
     my ( $self, $p ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     $log->is_info &&
         $log->info( "Indexing [", ref $self, ": ", $self->id, "]" );
     my $indexable = _indexable_object_text( $self, $p );
     my $wc = _tokenize( $self, $indexable );
     $log->is_debug &&
-        $log->debug( "Found the following tokens:", CTX->dump( $wc ) );
+        $log->debug( "Found tokens: ", join( ', ', keys %{ $wc } ) );
 
     # In the 'real world', we'd start a transaction here...
 
@@ -203,7 +205,7 @@ sub reindex_object {
 
 sub remove_object_from_index {
     my ( $self, $p ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $object_key = $self->generate_object_key;
     $log->is_info &&
@@ -226,7 +228,7 @@ sub remove_object_from_index {
 
 sub _indexable_object_text {
     my ( $self, $p ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     $p ||= {};
 
@@ -289,7 +291,7 @@ sub _tokenize {
 
 sub _store_terms {
     my ( $self, $terms ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $object_key = $self->generate_object_key;
     my $fields = join( ',', @SQL_FIELD_NAMES );
@@ -333,7 +335,7 @@ sub _store_terms {
 
 sub _execute_fulltext_search {
     my ( $class, $p ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $sql = $class->_build_sql_search_statement( $p->{terms} );
 
@@ -409,7 +411,7 @@ sub _fetch_raw_fulltext_results {
 
 sub _screen_results {
     my ( $class, $results, $p ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     # If this was an AND search, knock off all the results that didn't
     # have matches for all the terms
@@ -660,7 +662,7 @@ None known.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Chris Winters. All rights reserved.
+Copyright (c) 2001-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
