@@ -1,24 +1,29 @@
 package OpenInteract2::Page;
 
-# $Id: Page.pm,v 1.5 2003/06/25 14:10:49 lachoy Exp $
+# $Id: Page.pm,v 1.11 2004/05/11 04:30:59 lachoy Exp $
 
 use strict;
+use vars qw( %STORAGE_CLASS );
 use File::Basename           qw();
 use Log::Log4perl            qw( get_logger );
 use OpenInteract2::Constants qw( :log );
 use OpenInteract2::Context   qw( CTX );
 use OpenInteract2::Exception qw( oi_error );
+use OpenInteract2::Page::File;
 
-$OpenInteract2::Page::VERSION = sprintf("%d.%02d", q$Revision: 1.5 $ =~ /(\d+)\.(\d+)/);
+@OpenInteract2::Page::ISA     = qw( OpenInteract2::PagePersist );
+$OpenInteract2::Page::VERSION = sprintf("%d.%02d", q$Revision: 1.11 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 use constant DEFAULT_EXTENSION => '.html';
 
 sub FUNNY_CHARACTERS { return '\\\'"!#\$\%\|&\^\*\<\>{}\[\]\(\)\?' }
 
-my %STORAGE_CLASS = (
-     database => 'OpenInteract2::Page::Database',
-     file     => 'OpenInteract2::Page::File',
-     http     => 'OpenInteract2::Page::Http',
+%STORAGE_CLASS = (
+    database => 'OpenInteract2::Page::Database',
+    file     => 'OpenInteract2::Page::File',
+    http     => 'OpenInteract2::Page::Http',
 );
 
 
@@ -31,6 +36,31 @@ sub _class_initialize {
 # CLASS METHODS
 ########################################
 
+# override this so we can always make it an object-based call,
+# therefore including the 'location' value for the hierarchy checking;
+# unfortunately this means we always fetch it twice...
+
+sub check_action_security {
+    my ( $class, $params ) = @_;
+    if ( ref( $class ) ) {
+        return $class->SUPER::check_action_security( $params );
+    }
+    my $object = $class->fetch( $params->{id}, { skip_security => 1 } );
+    return $object->check_action_security( $params );
+}
+
+sub fetch_by_location {
+    my ( $class, $location ) = @_;
+    $log ||= get_logger( LOG_APP );
+    $log->is_debug &&
+        $log->debug( "Fetching page object by location '$location'" );
+    my $items = $class->fetch_group({ where => 'location = ?',
+                                      value => [ $location ] });
+    $log->is_debug &&
+        $log->debug( "Number of matching objects: ", ( ref $items ) ? scalar @{ $items } : 0 );
+    return $items->[0];
+}
+
 # Find object with $location in the database. We also see if the
 # request is a directory index request and try to map it to a
 # page_directory object
@@ -39,10 +69,10 @@ sub _class_initialize {
 
 sub page_by_location {
     my ( $class, $location ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     $log->is_debug &&
-        $log->debug( "Trying to retrieve object with location [$location]" );
+        $log->debug( "Trying to retrieve object with location '$location'" );
 
     # If someone requests an empty location (e.g., 'http://blah/')
     # then treat it as a root directory request.
@@ -53,7 +83,7 @@ sub page_by_location {
     # just try and fetch what was passed in
 
     if ( $class->CONFIG->{strict_location} ) {
-        return $class->fetch( $location );
+        return $class->fetch_by_location( $location );
     }
 
     # Onto the fun stuff.
@@ -78,10 +108,10 @@ sub page_by_location {
 
     if ( $no_extension ) {
         $log->is_debug &&
-            $log->debug( "Testing [$location] as a page with no extension" );
-        $page = eval { $class->fetch( $location ) };
+            $log->debug( "Testing '$location' as a page with no extension" );
+        $page = eval { $class->fetch_by_location( $location ) };
         if ( $@ ) {
-            $log->error( "Caught error fetching [$location] as page: $@" );
+            $log->error( "Caught error fetching '$location' as page: $@" );
         }
         elsif ( $page ) {
             return $page;
@@ -98,7 +128,7 @@ sub page_by_location {
 
     if ( $dir_request or $no_extension ) {
         $log->is_debug &&
-            $log->debug( "Testing [$location] as a directory" );
+            $log->debug( "Testing '$location' as a directory" );
         my $dir = eval {
             CTX->lookup_object( 'page_directory' )
                ->fetch_by_directory( $location )
@@ -110,7 +140,7 @@ sub page_by_location {
         if ( $dir ) {
             $log->is_debug &&
                 $log->debug( "Directory object found ",
-                             "[$dir->{directory}]. Returning..." );
+                             "'$dir->{directory}'. Returning..." );
             return $dir;
         }
         $log->is_debug &&
@@ -132,7 +162,7 @@ sub page_by_location {
         push @locations, join( '', $location, $default_extension );
         $log->is_debug &&
             $log->debug( "Location has no extension; add check ",
-                         "for default extension [$locations[-1]]" ),
+                         "for default extension '$locations[-1]'" ),
     }
 
     # If it does have an extension, go ahead and add it to the
@@ -145,7 +175,7 @@ sub page_by_location {
         ( $sans_extension = $location ) =~ s/$extension$//;
         $log->is_debug &&
             $log->debug( "Also check location without the extension ",
-                         "using [$sans_extension]" );
+                         "using '$sans_extension'" );
         push @locations, $sans_extension;
     }
 
@@ -156,8 +186,8 @@ sub page_by_location {
     if ( $extension and $extension ne $default_extension ) {
         my ( $root, $other_ext ) = $location =~ m|^(.*)(\.\w+)$|;
         $log->is_debug &&
-            $log->debug( "Extension [$extension] is not the default, also try ",
-                         "[$default_extension]" );
+            $log->debug( "Extension '$extension' is not the default, also try ",
+                         "'$default_extension'" );
         push @locations, join( '', $root, $default_extension );
     }
 
@@ -169,17 +199,20 @@ LOCATION:
         next unless ( $location );
         next if ( $location =~ /^\.\w+$/ );
         $log->is_debug &&
-            $log->debug( "Trying to fetch location [$location]" );
-        $page = eval { CTX->lookup_object( 'page' )->fetch( $location ) };
+            $log->debug( "Trying to fetch location '$location'" );
+        $page = eval {
+            CTX->lookup_object( 'page' )
+               ->fetch_by_location( $location )
+        };
         if ( $@ ) {
-            $log->error( "Encountered error trying to retrieve [$location] ",
+            $log->error( "Encountered error trying to retrieve '$location' ",
                          "($@); continuing with other locations." );
             $error = "$@";
             next LOCATION;
         }
         if ( $page ) {
             $log->is_debug &&
-                $log->debug( "Found matching page: [$page->{location}]" );
+                $log->debug( "Found matching page: '$page->{location}'" );
             return $page;
         }
     }
@@ -255,7 +288,7 @@ sub clean_location {
 
 sub list_directories {
     my ( $class ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $directory_list = eval {
         $class->db_select({ from            => $class->CONFIG->{table_name},
@@ -375,45 +408,6 @@ sub mime_type_content {
     return $MAGIC->checktype_contents( $content );
 }
 
-
-########################################
-# RULES
-########################################
-
-# Here we add a ruleset so we can set the value for 'directory'
-# whenever we save the object
-
-sub ruleset_factory {
-    my ( $class, $rs_table ) = @_;
-    push @{ $rs_table->{pre_save_action} },    \&find_directory;
-    push @{ $rs_table->{post_save_action} },   \&save_content;
-    push @{ $rs_table->{post_remove_action} }, \&remove_content;
-    return __PACKAGE__;
-}
-
-
-sub find_directory {
-    my ( $self, $p ) = @_;
-    $self->{location} =~ s|/+|/|g;
-    $self->{directory} = File::Basename::dirname( $self->{location} );
-    return 1;
-}
-
-
-sub save_content {
-    my ( $self, $p ) = @_;
-    return 1 unless ( $self->{content} );
-    my $storage_class = $STORAGE_CLASS{ $self->{storage} };
-    return $self->{content} = $storage_class->save( $self, $self->{content} );
-}
-
-
-sub remove_content {
-    my ( $self, $p ) = @_;
-    my $storage_class = $STORAGE_CLASS{ $self->{storage} };
-    return $self->{content} = $storage_class->remove( $self );
-}
-
 1;
 
 __END__
@@ -577,7 +571,7 @@ L<File::Basename|File::Basename>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Chris Winters. All rights reserved.
+Copyright (c) 2001-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
