@@ -1,6 +1,6 @@
 package OpenInteract2::SQLInstall::Group;
 
-# $Id: Group.pm,v 1.4 2003/06/25 14:10:49 lachoy Exp $
+# $Id: Group.pm,v 1.8 2004/02/25 16:05:16 lachoy Exp $
 
 use strict;
 use base qw( OpenInteract2::SQLInstall );
@@ -21,6 +21,8 @@ my %FILES = (
                 'sys_group_generator.sql' ],
 );
 
+my ( $log );
+
 sub get_structure_set {
     return 'group';
 }
@@ -41,6 +43,16 @@ sub get_structure_file {
     }
 }
 
+sub get_migration_information {
+    my ( $self ) = @_;
+    my %group_info = ( spops_class => 'OpenInteract2::Group' );
+    my %link_info = (
+        table => 'sys_group_user',
+        field => [ qw/ group_id user_id / ],
+    );
+    return [ \%group_info, \%link_info ];
+}
+
 sub get_security_file {
     return 'install_security.dat';
 }
@@ -56,16 +68,17 @@ sub install_data {
 
 sub _create_initial_groups {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_INIT );
+    $log ||= get_logger( LOG_INIT );
+
+    my $defaults = CTX->lookup_default_object_id;
 
-    my $server_config = CTX->server_config;
     my $group_class = CTX->lookup_object( 'group' );
     my @DATA = (
-        [ $server_config->{default_objects}{supergroup},
+        [ $defaults->{supergroup},
           'admin',      'Super group, can do anything' ],
-        [ $server_config->{default_objects}{public_group},
+        [ $defaults->{public_group},
           'public',     'All users should be part of this group' ],
-        [ $server_config->{default_objects}{site_admin_group},
+        [ $defaults->{site_admin_group},
           'site admin', 'Group for site administrators' ],
     );
 
@@ -97,10 +110,10 @@ sub _create_initial_groups {
 
 sub _create_link_groups {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_INIT );
+    $log ||= get_logger( LOG_INIT );
 
     my $action_name = 'create superuser group link';
-    my $server_config = CTX->server_config;
+    my $defaults = CTX->lookup_default_object_id;
     my $dbh = CTX->lookup_object( 'group' )
                  ->global_datasource_handle();
     my $sql = qq/
@@ -110,8 +123,8 @@ sub _create_link_groups {
     my ( $sth );
     eval {
         $sth = $dbh->prepare( $sql );
-        $sth->execute( $server_config->{default_objects}{supergroup},
-                       $server_config->{default_objects}{superuser} );
+        $sth->execute( $defaults->{supergroup},
+                       $defaults->{superuser} );
     };
     if ( $@ ) {
         $log->error( "Failed to create user/group link: $@" );
