@@ -1,6 +1,6 @@
 package OpenInteract2::User;
 
-# $Id: User.pm,v 1.4 2003/06/25 14:10:50 lachoy Exp $
+# $Id: User.pm,v 1.8 2004/02/22 04:38:45 lachoy Exp $
 
 use strict;
 use Log::Log4perl            qw( get_logger );
@@ -8,8 +8,12 @@ use OpenInteract2::Constants qw( :log );
 use OpenInteract2::Context   qw( CTX );
 use OpenInteract2::Exception qw( oi_error );
 use SPOPS::Secure            qw( :level :scope );
+use SPOPS::Utility;
 
-$OpenInteract2::User::VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
+@OpenInteract2::User::ISA     = qw( OpenInteract2::UserPersist );
+$OpenInteract2::User::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 ########################################
 # CLASS METHODS
@@ -21,10 +25,37 @@ sub fetch_by_login_name {
     my $user_list = $class->fetch_group({ where => "login_name $oper ?",
                                           value => [ $login_name ],
                                           %{ $p } });
-    if ( $p->{return_single} ) {
-        return $user_list->[0];
+    if ( $p->{return_multiple} ) {
+        return $user_list;
     }
-    return $user_list;
+    return $user_list->[0];
+}
+
+sub fetch_by_email {
+    my ( $class, $email, $p ) = @_;
+    $p ||= {};
+    my $oper = $class->sql_case_insensitive;
+    my $user_list = $class->fetch_group({ where => "email $oper ?",
+                                          value => [ $email ],
+                                          %{ $p } });
+    if ( $p->{return_multiple} ) {
+        return $user_list;
+    }
+    return $user_list->[0];
+}
+
+sub generate_password {
+    my ( $class, $options ) = @_;
+    my $length = $options->{length} || 8;
+    my $password = SPOPS::Utility->generate_random_code( $length );
+    my ( $crypted );
+    if ( $options->{crypt} ) {
+        $crypted = SPOPS::Utility->crypt_it( $password );
+    }
+    else {
+        $crypted = $password;
+    }
+    return ( $password, $crypted );
 }
 
 
@@ -33,7 +64,7 @@ sub fetch_by_login_name {
 
 sub make_public {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     my $group_class = CTX->lookup_object( 'group' );
 
@@ -76,7 +107,7 @@ sub full_name {
 
 sub increment_login {
     my ( $self ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     $self->{last_login} = $self->now;
     $self->{num_logins}++;
@@ -92,7 +123,7 @@ sub increment_login {
 
 sub check_password {
     my ( $self, $check_pw ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     return undef unless ( $check_pw );
     my $exist_pw = $self->{password};
@@ -157,9 +188,29 @@ B<fetch_by_login_name( $login_name, [ \%params ] )>
 
 Retrieve a single user by C<$login_name>. This performs a
 case-insensitive search (if available). Pass a true value for
-'return_single' in C<\%params> to return a single object, and pass a
-true value for 'skip_security' in C<\%params> to skip security
-checks. (Useful when finding a user to login with.)
+'return_multiple' in C<\%params> to return a multiple objects -- there
+should not be since the default schema shipped with OI2 declares this
+column 'UNIQUE'; and pass a true value for 'skip_security' in
+C<\%params> to skip security checks. (Useful when finding a user to
+login with.)
+
+B<fetch_by_email( $email, [ \%params ] )>
+
+Return a single user with email C<$email>. This performs a
+case-insensitive search (if available). Pass a true value for
+'return_multiple' in C<\%params> to return multiple objects -- there
+should not be since the default schema shipped with OI2 declares this
+column 'UNIQUE'.
+
+B<generate_password( \%options )>
+
+Generate a random password. Valid options are C<crypt> (set to true
+value to encrypt) and C<length> (set to the length of the password to
+generate).
+
+Returns a two-item list: the first item is the uncrypted password, the
+second item is the crypted password. If you did not specify a true
+value for 'crypt' they will be the same.
 
 =head2 Object Methods
 
@@ -195,7 +246,7 @@ None known.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Chris Winters. All rights reserved.
+Copyright (c) 2001-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
