@@ -1,14 +1,18 @@
 package OpenInteract2::User::LDAP;
 
-# $Id: LDAP.pm,v 1.2 2003/06/25 14:10:50 lachoy Exp $
+# $Id: LDAP.pm,v 1.7 2004/02/22 04:48:55 lachoy Exp $
 
 use strict;
+use base qw( OpenInteract2::User );
+
 use Log::Log4perl            qw( get_logger );
 use OpenInteract2::Constants qw( :log );
 use OpenInteract2::Context   qw( CTX );
 use OpenInteract2::Exception qw( oi_error );
 
-$OpenInteract2::User::LDAP::VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::User::LDAP::VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 ########################################
 # Normal
@@ -24,7 +28,7 @@ sub full_name {
 
 sub check_password {
     my ( $self, $check_passwd ) = @_;
-    my $log = get_logger( LOG_APP );
+    $log ||= get_logger( LOG_APP );
 
     # If we're using multiple datasources
     # (SPOPS::LDAP::MultiDatasource) then the name of the datasource
@@ -60,7 +64,11 @@ sub fetch_by_login_name {
     my $login_field = $class->CONFIG->{field_map}{login_name}
                       || 'login_name';
     $p->{filter} = "$login_field=$login_name";
-    return $class->fetch( undef, $p )
+    my $users = $class->fetch( undef, $p );
+    if ( $p->{return_multiple} ) {
+        return $users;
+    }
+    return $users->[0];
 }
 
 1;
