@@ -1,13 +1,15 @@
 package OpenInteract2::Config::Ini;
 
-# $Id: Ini.pm,v 1.9 2003/06/24 03:35:38 lachoy Exp $
+# $Id: Ini.pm,v 1.12 2004/02/18 05:25:27 lachoy Exp $
 
 use strict;
 use Log::Log4perl            qw( get_logger );
 use OpenInteract2::Constants qw( :log );
 use OpenInteract2::Context   qw( CTX );
 
-$OpenInteract2::Config::Ini::VERSION = sprintf("%d.%02d", q$Revision: 1.9 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::Config::Ini::VERSION = sprintf("%d.%02d", q$Revision: 1.12 $ =~ /(\d+)\.(\d+)/);
+
+my ( $log );
 
 ########################################
 # CLASS METHODS
@@ -22,14 +24,28 @@ sub new {
     my $class = ref $pkg || $pkg;
     my $self = bless( {}, $class );
     if ( $self->{_m}{filename} = $params->{filename} ) {
-        $self->translate_ini( OpenInteract2::Config->read_file( $params->{filename} ) );
+        $self->_translate_ini( OpenInteract2::Config->read_file( $params->{filename} ) );
     }
     elsif ( $params->{content} ) {
-        $self->translate_ini( $params->{content} );
+        $self->_translate_ini( $params->{content} );
+    }
+    elsif ( $params->{struct} ) {
+        $self->_translate_struct_to_ini( $params->{struct} );
     }
     return $self;
 }
 
+# Return the INI data as a raw hashref (naive for now...)
+
+sub as_data {
+    my ( $self ) = @_;
+    my %data = ();
+    while ( my ( $k, $v ) = each %{ $self } ) {
+        $data{ $k } = $v unless ( $k eq '_m' );
+    }
+    return \%data;
+}
+
 
 sub get {
     my ( $self, $section, @p ) = @_;
@@ -50,7 +66,7 @@ sub set {
     unless ( $self->is_section( $section, $sub_section ) ) {
         $self->add_section( $section, $sub_section );
     }
-    return $self->read_item( $section, $sub_section, $param, $value );
+    return $self->_read_item( $section, $sub_section, $param, $value );
 }
 
 
@@ -117,9 +133,9 @@ sub get_comments {
 # INPUT
 ########################################
 
-sub translate_ini {
+sub _translate_ini {
     my ( $self, $content ) = @_;
-    my $log = get_logger( LOG_CONFIG );
+    $log ||= get_logger( LOG_CONFIG );
 
     # Content can be either an arrayref of lines from a file or a
     # scalar with the content.
@@ -132,6 +148,9 @@ sub translate_ini {
     my @comments = ();
     my ( $section, $sub_section );
 
+    my ( $multiline_value, $multiline_param );
+    my $in_multiline = 0;
+
     # Cycle through the lines: skip blanks; accumulate comments for
     # each section; register section/subsection; add parameter/value
 
@@ -142,30 +161,62 @@ sub translate_ini {
             next;                 # ... get rid of current line
         }
         s/\s+$//;
+
         if ( /^\s*\#/ ) {
             push @comments, $_;
+            $multiline_param = undef;
+            $multiline_value = undef;
+            $in_multiline = 0;
             next;
         }
+
         if ( /^\s*\[\s*(\S|\S.*\S)\s*\]\s*$/) {
             $log->is_debug &&
                 $log->debug( "Found section [$1]" );
             ( $section, $sub_section ) =
-                              $self->read_section_head( $1, \@comments );
+                              $self->_read_section_head( $1, \@comments );
             @comments = ();
             next;
         }
-        my ( $param, $value ) = /^\s*([^=]+?)\s*=\s*(.*)\s*$/;
+        my ( $param, $value );
+
+        my $this_multiline = $_ =~ s|\\$||;
+        if ( $in_multiline and $this_multiline ) {
+            $multiline_value .= "$_\n";
+            next;
+        }
+        elsif ( $in_multiline ) {
+            $param = $multiline_param;
+            $value = $multiline_value . $_;
+            $multiline_param = undef;
+            $multiline_value = undef;
+            $in_multiline = 0;
+        }
+        else {
+            ( $param, $value ) = /^\s*([^=]+?)\s*=\s*(.*)\s*$/;
+            if ( $this_multiline ) {
+                $multiline_param = $param;
+                $multiline_value = $value;
+                $in_multiline = 1;
+                next;
+            }
+        }
+
         $log->is_debug &&
             $log->debug( "Set [$section $sub_section $param] ",
                          "to [$value]" );
-        $self->read_item( $section, $sub_section, $param, $value );
+        $self->_read_item( $section, $sub_section, $param, $value );
     }
     return $self;
+}
 
+sub _translate_struct_to_ini {
+    my ( $self, $struct ) = @_;
+    die "_translate_struct_to_ini() not done yet!";
 }
 
 
-sub read_section_head {
+sub _read_section_head {
     my ( $self, $full_section, $comments ) = @_;
     my $comment_text = join "\n", @{ $comments };
     if ( $full_section =~ /^([\w\-]+)\s+([\w\-]+)$/ ) {
@@ -182,14 +233,14 @@ sub read_section_head {
 }
 
 
-sub read_item {
+sub _read_item {
     my ( $self, $section, $sub_section, $param, $value ) = @_;
 
     # Special case -- 'Global' stuff goes in the config object root
 
     if ( $section eq 'Global' ) {
         push @{ $self->{_m}{global} }, $param;
-        $self->set_value( $self, $param, $value );
+        $self->_set_value( $self, $param, $value );
         return;
     }
 
@@ -199,12 +250,12 @@ sub read_item {
     }
 
     return ( $sub_section )
-             ? $self->set_value( $self->{ $section }{ $sub_section }, $param, $value )
-             : $self->set_value( $self->{ $section }, $param, $value );
+             ? $self->_set_value( $self->{ $section }{ $sub_section }, $param, $value )
+             : $self->_set_value( $self->{ $section }, $param, $value );
 }
 
 
-sub set_value {
+sub _set_value {
     my ( $self, $set_in, $param, $value ) = @_;
     my $existing = $set_in->{ $param };
     if ( $existing and ref $set_in->{ $param } eq 'ARRAY' ) {
@@ -224,7 +275,7 @@ sub set_value {
 
 sub write_file {
     my ( $self, $filename ) = @_;
-    my $log = get_logger( LOG_CONFIG );
+    $log ||= get_logger( LOG_CONFIG );
 
     $filename ||= $self->{_m}{filename} || 'config.ini';
     my ( $original_filename );
@@ -252,7 +303,7 @@ sub write_file {
             print OUT "$comments\n";
         }
         print OUT "[$full_section]\n",
-                  $self->output_section( $section, $sub_section ),
+                  $self->_output_section( $section, $sub_section ),
                   "\n\n";
     }
     close( OUT );
@@ -265,7 +316,7 @@ sub write_file {
 }
 
 
-sub output_section {
+sub _output_section {
     my ( $self, $section, $sub_section ) = @_;
     my $show_from = ( $sub_section )
                       ? $self->{ $section }{ $sub_section }
@@ -274,21 +325,21 @@ sub output_section {
     foreach my $key ( keys %{ $show_from } ) {
         if ( ref $show_from->{ $key } eq 'ARRAY' ) {
             foreach my $value ( @{ $show_from->{ $key } } ) {
-                push @items, $self->show_item( $key, $value );
+                push @items, $self->_show_item( $key, $value );
             }
         }
         elsif ( ref $show_from->{ $key } eq 'HASH' ) {
             # no-op -- this should get picked up later
         }
         else {
-            push @items, $self->show_item( $key, $show_from->{ $key } );
+            push @items, $self->_show_item( $key, $show_from->{ $key } );
         }
     }
     return join "\n", @items;
 }
 
 
-sub show_item { return join( ' = ', $_[1], $_[2] ) }
+sub _show_item { return join( ' = ', $_[1], $_[2] ) }
 
 1;
 
@@ -301,15 +352,16 @@ OpenInteract2::Config::Ini - Read/write INI-style (++) configuration files
 =head1 SYNOPSIS
 
  my $config = OpenInteract2::Config::Ini->new({ filename => 'myconf.ini' });
- print "Main database driver is:", $config->{db_info}{main}{driver}, "\n";
- $config->{db_info}{main}{username} = 'mariolemieux';
+ print "Main database driver is:", $config->{datasource}{main}{driver}, "\n";
+ $config->{datasource}{main}{username} = 'mariolemieux';
  $config->write_file;
 
 =head1 DESCRIPTION
 
 This is a very simple implementation of a configuration file
 reader/writer that preserves comments and section order, enables
-multivalue fields and one or two-level sections.
+multivalue fields, enables multi-line values, and has one or two-level
+sections.
 
 Yes, there are other configuration file modules out there to
 manipulate INI-style files. But this one takes several features from
@@ -353,6 +405,8 @@ Given the following configuration in INI-style:
  sql_install   =
  long_read_len = 65536
  long_trunc_ok = 0
+ comment       = this is the database for mr whitman who \
+ is not feeling very well as of late
 
  [db_info other]
  db_owner      =
@@ -383,6 +437,7 @@ You would get the following Perl data structure:
            sql_install   => undef,
            long_read_len => '65536',
            long_trunc_ok => '0',
+           comment       => 'this is the database for mr whitman who is not feeling very well as of late',
       },
       other => {
            db_owner      => undef,
@@ -417,14 +472,64 @@ will be available as:
 B<new( \%params )>
 
 Create a new configuration object. If you pass in 'filename' as a
-parameter we will try to read it in using C<read_file()>. If you pass
-in 'content' as a parameter we try to translate it using
-C<translate_ini()>
+parameter we will parse the file and fill the returned object with its values.
 
-Returns: a new C<OpenInteract2::Config::Ini> object
+If you pass in raw INI text in the parameter 'content' we try to
+translate it using the same means as reading a file.
+
+And if you pass in a hashref in the parameter 'struct' we attempt to
+map its keys and values to the internal format which can then be saved
+as normal. This will throw an exception if your structures are nested
+too deeply. For instance, this would be ok:
+
+ my $foo = {
+    top_key => { myvalue => 1, yourvalue => [ 'one', 'two' ] },
+    bottom_key => { other => { mine => '1', yours => 2 }, bell => 'weather' },
+ };
+
+As it would represent:
+
+ [top_key]
+ myvalue = 1
+ yourvalue = one
+ yourvalue = two
+
+ [bottom_key]
+ bell = weather
+
+ [bottom_key other]
+ mine = 1
+ yours = 2
+
+But the following has references nested too deeply:
+
+ my $foo = {
+    top_key => {
+        myvalue => 1,
+        yourvalue => [ 'one', 'two' ]
+    },
+    bottom_key => {
+        other => {
+            mine => {              <--- this key's value is too deep
+                zaphod => 'towel',
+            },
+            yours => {             <--- this key's value is too deep
+               abe => 'honest',
+            },
+        }
+        bell => 'weather',
+    },
+ };
+
+Returns: a new
+L<OpenInteract2::Config::Ini|OpenInteract2::Config::Ini> object
 
 =head2 Object Methods
 
+B<as_data()>
+
+Get the data back from the object as an unblessed hash reference.
+
 B<sections()>
 
 Returns a list of available sections.
@@ -451,49 +556,6 @@ Remove the C<$section> (and C<$sub_section>, if given) entirely.
 
 Returns: the value deleted
 
-=head2 Input Methods
-
-B<read_file( $filename )>
-
-Reads content from C<$filename>, translates it and puts it into the object.
-
-Returns: the object filled with the content.
-
-B<translate_ini( \@lines|$content )>
-
-Translate the arrayref C<\@lines> or the scalar C<content> from INI
-format into a Perl data structure.
-
-Returns: the object filled with the content.
-
-B<read_section_head( $full_section, \@comments )>
-
-Splits the section into a section and sub-section, returning a
-two-item list. Also puts the full section in the object internal order
-and puts the comments so they can be linked to the section.
-
-Returns: a two-item list with the section and sub-section as
-elements. If the section is only one-level deep, it is the first and
-only member.
-
-B<read_item( $section, $subsection, $parameter, $value )>
-
-Reads the value from [C<$section> C<$subsection>] into the object. If
-the C<$section> is 'Global' we set the C<$parameter> and C<$value>
-at the root level.
-
-Returns: the value set
-
-B<set_value( \%values, $parameter, $value )>
-
-Sets C<$parameter> to C<$value> in C<\%values>. We do not care where
-C<\%values> is in the tree.
-
-If a value already exists for C<$parameter>, we make the value of
-C<$parameter> an arrayref and push C<$value> onto it.
-
-=head2 Output Methods
-
 B<write_file( $filename )>
 
 Serializes the INI file (with comments, as applicable) to
@@ -503,26 +565,61 @@ Items from the config object root go into 'Global'.
 
 Returns: the filename to which the INI structure was serialized.
 
-B<output_section( $section, $sub_section )>
+=head2 Debugging Note
+
+Configuration input and output can generate a ton of logging
+information, so it uses a separate logging category 'LOG_CONFIG' as
+imported from
+L<OpenInteract2::Constants|OpenInteract2::Constants>. Set this to
+C<DEBUG> with fair warning...
+
+=head2 Internal Methods
+
+B<_translate_ini( \@lines|$content )>
+
+Translate the arrayref C<\@lines> or the scalar C<content> from INI
+format into a Perl data structure.
+
+Returns: the object filled with the content.
+
+B<_read_section_head( $full_section, \@comments )>
+
+Splits the section into a section and sub-section, returning a
+two-item list. Also puts the full section in the object internal order
+and puts the comments so they can be linked to the section.
+
+Returns: a two-item list with the section and sub-section as
+elements. If the section is only one-level deep, it is the first and
+only member.
+
+B<_read_item( $section, $subsection, $parameter, $value )>
+
+Reads the value from [C<$section> C<$subsection>] into the object. If
+the C<$section> is 'Global' we set the C<$parameter> and C<$value>
+at the root level.
+
+Returns: the value set
+
+B<_set_value( \%values, $parameter, $value )>
+
+Sets C<$parameter> to C<$value> in C<\%values>. We do not care where
+C<\%values> is in the tree.
+
+If a value already exists for C<$parameter>, we make the value of
+C<$parameter> an arrayref and push C<$value> onto it.
+
+B<_output_section( $section, $sub_section )>
 
 Serializes the section C<$section> and C<$sub_section>.
 
 Returns: a scalar suitable for output.
 
-B<show_item( $parameter, $value )>
+B<_show_item( $parameter, $value )>
 
 Serialize the key/value pair.
 
 Returns: "$parameter = $value"
 
-=head2 Debugging Note
-
-Because this and other debugging classs can produce so much
-information, it uses a separate debug switch than the rest of OI. If
-you want to see debugging from this class, call:
-
- OpenInteract2::Config->SET_DEBUG(1);
-
 =head1 SEE ALSO
 
 L<AppConfig|AppConfig>
@@ -533,11 +630,11 @@ L<Config::IniFiles|Config::IniFiles>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Chris Winters. All rights reserved.
+Copyright (c) 2001-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =head1 AUTHORS
 
-Chris Winters E<lt>chris@cwinters.comE<gt>
+Chris Winters E<lt>chris@cwinters.comE<gt>
\ No newline at end of file
